/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* ToughBuilt ClipTech Receiver
* Benjamen Johnson <workshop.electronsmith.com>
* 20201108
*
* openSCAD Version: 2019.05 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
version = "V1.0";

// A bunch of definitions
RECEIVER=0;SPRING=1;PIN=2;COUNTERBORE=0;NUT=1;$dl =0.01;
INVERT_X = [[-1,0,0],[0,1,0],[0,0,1]];

/*[Edit ME]*/
/*******************************************************************************
*
*******************************************************************************/
// Which part do you want?
part = 0; //[0:Receiver, 1: Spring, 2: Pin]

//Do you want a counter bore or relief for nuts?
hole_type = 0; //[0:CounterBore,1:Nut]

// Adjust base of hub to change the fit of the receiver
hub_base_depth = 3.8; //[3:0.1:4]

// Adjust trapezoidal part of the hub to change the fit of the receiver
hub_depth = 5; //[4.5:0.1:5.5]

/*[Hub Parameters]*/
/*******************************************************************************
*        
*         |- top-| 
*          ------      ---
*        /        \     |
*       /          \    hub_height
*      /            \   |
*     /              \  |
*     ---------------- ---
*    |---- bottom ----|
*
*         
*          ------     --- top_gap
*        /  ----  \   ---  
*       /  |    |  \
*      /   |    |   \
*     /    |    |    \
*     ----------------
*          |    |
*        base_width
*
*******************************************************************************/
// Length of the base of the trapezoid part of the clip
hub_bottom = 46.4;

// Length of the top of the trapezoid part of the clip
hub_top = 24;

// Height of the trapezoid part of the clip
hub_height = 50.4;

// Space at top where base doesn't extend
hub_top_gap = 3.75;

// How far down fom the top of the clip is the latch indent
latch_drop = 8.25;

// distance to jump over from axis to start top 
top_start = (hub_bottom-hub_top)/2;

// Width of the hub base
hub_base_width = 22.5;

// height of the hub base
hub_base_height = hub_height-hub_top_gap;

// define the hub base
HUB_BASE_DEF = [hub_base_width,hub_base_depth+$dl,hub_base_height];

//define the points for trapezoid part of the clip
HUB = [[0,0],[hub_bottom,0],[top_start+hub_top,hub_height],[top_start, hub_height]];

// matrix to move the clip  to the center;
CENTER =   [[-hub_bottom/2,-hub_height/2],
            [-hub_bottom/2,-hub_height/2],
            [-hub_bottom/2,-hub_height/2],
            [-hub_bottom/2,-hub_height/2]];

// center the CLIP
CENTERED_HUB = HUB+CENTER;

/*[Receiver Parameters]*/
/*******************************************************************************
*
*******************************************************************************/
// How thick the total reciever body is
receiver_depth = 15;

// wall thickness of the reciever
wall_thickness = 3;

// Thickness of the back plate
plate_thickness = 3;

// Width of the back plate
plate_width = 80;

// How far does the plate extend above the reciever
plate_overhang = 12;

// Calculate the actual plate height
plate_height =hub_height+2*wall_thickness+plate_overhang;

// how much of the corner to cut off the top of the plate
plate_chamfer = 20;

// Vector that defines the plate size
PLATE_DEF = [plate_width,plate_thickness,plate_height];

// location of the plate
PLATE_POS = [0,receiver_depth-plate_thickness/2,plate_overhang/2];

/*[Pin Parameters]*/
/*******************************************************************************
*
*******************************************************************************/
// Radius of the pin
pin_r = 2.5;

// Length of the pin
pin_h = 8+$dl;

// Where the pin is located 
pin_pos = 8;

/*[Spring Parameters]*/
/*******************************************************************************
*
*******************************************************************************/
// amount of space to leave around the spring
spring_clearance = 0.5; //[0.1:0.1:1]

spring_width = 10;
spring_thickness = 3;
dovetail_angle = 30;

spring_fit = 1.01;

// Length of retention spring
spring_length = 30;

// set the ball detent diameter and position
detent_dia = 5;

// Widest part of the dovetail
spring_base = spring_width+2*spring_thickness*tan(dovetail_angle);

// define the points of a trapezoid (dovetail)
SPRING_DEF =[[-spring_width/2,-spring_thickness/2],
        [spring_width/2,-spring_thickness/2],
        [spring_base/2,spring_thickness/2],
        [-spring_base/2,spring_thickness/2]];        

/*[Hole Parameters]*/
/*******************************************************************************
*
******************************************************************************/
// Reduce height of the reciever 
height_adj = 0;//[-15:1:0]

// horizontal spacing between the holes
h_hole_spacing = 60;

// vertical spacing between the holes(0 for two holes)
v_hole_spacing = 25;

//Vertical hole position (from top)
vert_hole_pos = 0;

// How deep the hole is (how much material under the screw?)
hole_depth=5;

// Diameter of the hole
hole_dia = 5;

// Diameter of the counterbore
counter_dia = 10;

// What is the wrench size of the hex nut to be pressed
hex_nut_size = 8.62;

// Adjust the pressed hex nut fit
hex_nut_fit = 1; //[.90:0.01:1.1]

// How deep should the hex nut sit in the strut nut?
hex_nut_depth = 4;

// Calculate the point to point diameter of the pressed nut
pt_to_pt_hex_nut_dia = 2*(hex_nut_size/2)/cos(30)* hex_nut_fit;

HOLE_POS = [-h_hole_spacing/2,receiver_depth-hole_depth,plate_height-wall_thickness+vert_hole_pos];

/*[Stamp Options]*/
/*******************************************************************************
*
******************************************************************************/
// Which font do you want to use
stamp_font = "Arial:style=Bold";

// How big you want the text
font_size = 10;//[0:0.5:8]

// How thick the font is (back to front)
font_thickness = 0.4;

/*******************************************************************************
* Make the receiver
*******************************************************************************/
// make recever the right height
if(part == RECEIVER)
    difference(){
        translate([0,0,height_adj])
        make_reciever();
        
        //cutoff bottom
        translate([0,0,-10])
        cube([100,100,20],center = true);
        
        // Version Stamp
        translate([0,receiver_depth-font_thickness+$dl,font_size/2+3])
        rotate([90,0,180])
        stamp_version(version,font_size,font_thickness);
    } // end difference
else if(part == SPRING)
    translate([0,0,0])
    rotate([0,0,0])
    spring();
else if(part == PIN)
    rotate([0,90,0])
    cylinder(d=5,h=8,$fn=20,center=true);

/*******************************************************************************
* Module for making the clip
*******************************************************************************/
module make_reciever(){ 
    difference(){  
        //make receiver body
        translate([0,0,(hub_height)/2])
        hull(){
            difference(){
                union(){
                    rotate([90,0,180])    
                    linear_extrude(height = receiver_depth,convexity=10) 
                        offset(delta = wall_thickness)polygon(CENTERED_HUB);
                            
                    // make tabs
                    translate(PLATE_POS)
                        cube(PLATE_DEF,center = true);
                }//end union
                
            // cut the corners off the top of the plate    
            translate([plate_width/2,15,hub_height/2+15])
            rotate([0,45,0])
            cube([plate_chamfer,30,plate_chamfer],center=true);
            
            translate([-plate_width/2,15,hub_height/2+15])
            rotate([0,45,0])
            cube([plate_chamfer,30,plate_chamfer],center=true);
    
            }//end difference
        }//end hull 
        make_hub();

        // cut out slot for spring
        scale([spring_fit,1,1]){
        translate([0,hub_base_depth+5+hub_depth-$dl,hub_height-latch_drop])
            cube([spring_base,10,15],center = true);
        
        // cut slot for inserting spring
        translate([0,hub_base_depth+spring_thickness/2+hub_depth-$dl,0])
        linear_extrude(height=plate_height,convexity=10)
            polygon(SPRING_DEF);
        }
        
        //cut mounting holes
        translate(HOLE_POS)
        screw_hole();
        
        translate(INVERT_X*HOLE_POS)
        screw_hole();
        
        translate(HOLE_POS-[0,0,v_hole_spacing])
        screw_hole();

        translate(INVERT_X*HOLE_POS-[0,0,v_hole_spacing])
        screw_hole();
        
        // cutout for spring pin
        translate([0,receiver_depth-4,plate_height-wall_thickness-pin_pos])
        rotate([-90,0,0])
        cylinder(r=pin_r,h=pin_h,center=true,$fn=20);           

    }// end difference   
    
}//end module
/*******************************************************************************
* Module for making the screw holes
*******************************************************************************/
module screw_hole(){
    rotate([-90,0,0])     
    union(){
        cylinder(d=hole_dia,h=20,center=true,$fn=20);
        
        if(hole_type == COUNTERBORE)
            translate([0,0,-5])
            cylinder(d=counter_dia,h=10,center=true,$fn=20);
        else
            translate([0,0,-5])
            cylinder(d=pt_to_pt_hex_nut_dia,h=10,center=true,$fn=6);           
    }//end union
} // end module

/*******************************************************************************
* Module for making the hub cutout
*******************************************************************************/
module make_hub(){
    union(){
        translate([0,hub_base_depth,hub_height/2-$dl])
        rotate([90,0,180])    
        linear_extrude(height = hub_depth) 
            polygon(CENTERED_HUB);
        
        translate([0,hub_base_depth/2,hub_base_height/2-$dl])
        cube(HUB_BASE_DEF,center=true);
    } //end union
} //end module

/*******************************************************************************
* Module for making the spring
*******************************************************************************/
module spring(x=1){
    
detent_pos = -(spring_length-detent_dia)/2;
  
// Create important vectors    
DETENT_POS = [0,detent_pos,spring_thickness/2];
difference(){    
    union(){
        rotate([-90,0,0])
        translate([0,0,-spring_length/2])
        linear_extrude(height=spring_length,convexity=10)
            polygon(SPRING_DEF);
        
        // Create hemisphere
        translate(DETENT_POS)
        rotate([0,90,0])
        cylinder(d=detent_dia, h=spring_width,center=true,$fn=20);
    }//end union
    translate([0,spring_length/2-pin_pos,0])
    cylinder(r=pin_r,h=pin_h,center=true,$fn=20);
}//end difference
} //end module

/*******************************************************************************
* Module for putting the version number on the print
*******************************************************************************/
module stamp_version(version_text,size,thickness){
    color("white")
    linear_extrude(height = thickness)
        text(text = version_text, font = stamp_font, size = size, valign="center",halign = "center");
} // end module